((nil
  (indent-tabs-mode . nil)
  (copyright-names-regexp
   . "\\(?:Jonas Bernoulli\\|Free Software Foundation, Inc\\.\\)"))
 (makefile-mode
  (indent-tabs-mode . t))
 (git-commit-mode
  (git-commit-major-mode . git-commit-elisp-text-mode))
 ("CHANGELOG"
  (nil (fill-column . 70)
       (mode . display-fill-column-indicator))))
